@@ -1,27 +1,27 @@
 """Helpers for model flattening."""
-import logging
 import os
 import time
 from pathlib import Path
 
 import libsbml
 
+from sbmlutils.console import console
 from sbmlutils.io import read_sbml, write_sbml
-from sbmlutils.utils import bcolors
+from sbmlutils.log import get_logger
 from sbmlutils.validation import validate_doc
 
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 def flatten_sbml(
-    sbml_path: Path, filepath=None, leave_ports=True
+    sbml_path: Path, sbml_flat_path: Path = None, leave_ports: bool = True
 ) -> libsbml.SBMLDocument:
     """Flatten given SBML file.
 
-    :param sbml_path: Comp SBML Path
-    :param filepath: SBML file to write to
-    :param leave_ports: flat to leave ports in flattened model.
+    :param sbml_path: input path to SBML file to flatten (should be a comp model)
+    :param sbml_flat_path: output path for flat SBML
+    :param leave_ports: boolean flag to leave ports in flattened model.
 
     :return: flattened SBMLDocument
     """
@@ -35,7 +35,9 @@ def flatten_sbml(
     os.chdir(str(sbml_path.parent))
 
     doc = read_sbml(source=sbml_path)
-    flat_doc = flatten_sbml_doc(doc, leave_ports=leave_ports, output_path=filepath)
+    flat_doc = flatten_sbml_doc(
+        doc, leave_ports=leave_ports, output_path=sbml_flat_path
+    )
 
     # change back the working dir
     os.chdir(working_dir)
@@ -71,7 +73,6 @@ def flatten_sbml_doc(
             logger.error("SBML errors in doc, see SBMLDocument error log.")
 
     # converter options
-    libsbml.CompFlatteningConverter
     props = libsbml.ConversionProperties()
     props.addOption("flatten comp", True)  # Invokes CompFlatteningConverter
     props.addOption("leave_ports", leave_ports)  # Indicates whether to leave ports
@@ -83,19 +84,19 @@ def flatten_sbml_doc(
     flattened_status = result == libsbml.LIBSBML_OPERATION_SUCCESS
 
     lines = [
-        "",
-        "-" * 120,
         str(doc),
         "{:<25}: {}".format("flattened", str(flattened_status).upper()),
         "{:<25}: {:.3f}".format("flatten time (ms)", time.perf_counter() - current),
-        "-" * 120,
     ]
-    info = bcolors.BOLD + "\n".join(lines) + bcolors.ENDC
+    info = "\n".join(lines)
 
     if flattened_status:
-        logger.info(bcolors.OKGREEN + info + bcolors.ENDC)
+        console.rule("Flatten SBML", style="success")
+        console.print(info, style="success")
+        console.rule(style="success")
     else:
-        logger.error(bcolors.FAIL + info + bcolors.ENDC)
+        console.rule("Flatten SBML", style="error")
+        console.print(info, style="error")
         raise ValueError(
             "SBML could not be flattend due to errors in the SBMLDocument."
         )
