@@ -39,17 +39,21 @@ Not supported:
 # TODO: rnd via dist (also normal)
 # TODO: rewrite using a proper parser like PLY Lex-Yacc (especially the function replacements are very cumbersome)
 
+import itertools
 import re
 import warnings
 from pathlib import Path
 from pprint import pprint
+from typing import Any, Dict, List, Optional, Tuple
 
 import libsbml
 
 from sbmlutils import __version__
 from sbmlutils import factory as fac
 from sbmlutils.converters import xpp_helpers
+from sbmlutils.factory import Event, Function
 from sbmlutils.io import sbml
+from sbmlutils.notes import NotesFormat
 
 
 XPP_ODE = "ode"
@@ -83,7 +87,7 @@ XPP_TYPE_CHARS = {
 NOTES = """
     <body xmlns='http://www.w3.org/1999/xhtml'>
     <h1>XPP model</h1>
-    <p>This model was converted from XPP ode format to SBML using <code>sbmlutils-{}</code>.</p>
+    <p>This model was converted from XPP ode format to SBML using <code>sbmlutils</code>.</p>
     <pre>{}</pre>
     <div class="dc:publisher">This file has been produced by
       <a href="https://github.com/matthiaskoenig/sbmlutils/" title="sbmlutils" target="_blank">sbmlutils</a>.
@@ -105,9 +109,7 @@ NOTES = """
              the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.</p>
       </div>
     </body>
-""".format(
-    __version__, "{}"
-)
+"""
 
 
 def escape_string(info: str) -> str:
@@ -118,7 +120,7 @@ def escape_string(info: str) -> str:
     return info
 
 
-def parse_keyword(xpp_id):
+def parse_keyword(xpp_id: str) -> Optional[str]:
     """Parse the keyword and returns the xpp keyword type.
 
     :param xpp_id:
@@ -128,11 +130,11 @@ def parse_keyword(xpp_id):
     for xpp_key, c in XPP_TYPE_CHARS.items():
         if xpp_id.startswith(c):
             return xpp_key
-    warnings.warn("Keyword not supported: {}".format(xpp_id))
+    warnings.warn(f"Keyword not supported: {xpp_id}")
     return None
 
 
-def parts_from_expression(expression):
+def parts_from_expression(expression: str) -> List[str]:
     """Return the parts of given expression.
 
     The parts can be whitespace or comma separated.
@@ -163,7 +165,7 @@ def parts_from_expression(expression):
     return parts
 
 
-def sid_value_from_part(part):
+def sid_value_from_part(part: str) -> Tuple[str, str]:
     """Get sid, value tuple from given part of expression.
 
     :param part:
@@ -182,9 +184,10 @@ def xpp2sbml(
     force_lower: bool = False,
     validate: bool = True,
     debug: bool = False,
-):
+) -> libsbml.SBMLDocument:
     """Read given xpp_file and converts to SBML file.
 
+    :param debug:
     :param xpp_file: xpp input ode file
     :param sbml_file: sbml output file
     :param force_lower: force lower case for all lines
@@ -214,10 +217,10 @@ def xpp2sbml(
         # mod (modulo)
         fac.Function("mod", "lambda(x,y, x % y)", name="modulo"),
     ]
-    function_definitions = []
-    events = []
+    function_definitions: List[Dict[str, Any]] = []
+    events: List[Event] = []
 
-    def replace_fdef():
+    def replace_fdef() -> bool:
         """Replace all arguments within the formula definitions."""
         changes = False
         for k, _fdata in enumerate(function_definitions):
@@ -245,12 +248,12 @@ def xpp2sbml(
 
         return changes
 
-    def create_initial_assignment(sid, value):
+    def create_initial_assignment(sid: str, value: str) -> None:
         """Create initial assignments helper."""
         # check if valid identifier
         if "(" in sid:
             warnings.warn(
-                "sid is not valid: {}. Initial assignment is not generated".format(sid)
+                f"sid is not valid: {sid}. Initial assignment is not generated"
             )
             return
 
@@ -260,21 +263,20 @@ def xpp2sbml(
                 fac.Parameter(
                     sid=sid,
                     value=f_value,
-                    name="{} = {}".format(sid, value),
+                    name=f"{sid} = {value}",
                     constant=False,
                 )
             )
         except ValueError:
             """
-            Initial data are optional, XPP sets them to zero by default (many xpp model don't write the p(0)=0.
+            Initial data are optional, XPP sets them to zero by default (many
+            xpp model don't write the p(0)=0.
             """
             parameters.append(
                 fac.Parameter(sid=sid, value=0.0, name=sid, constant=False)
             )
             initial_assignments.append(
-                fac.InitialAssignment(
-                    sid=sid, value=value, name="{} = {}".format(sid, value)
-                )
+                fac.InitialAssignment(sid=sid, value=value, name=f"{sid} = {value}")
             )
 
     ###########################################################################
@@ -287,7 +289,7 @@ def xpp2sbml(
 
         # add info to sbml
         text = escape_string("".join(lines))
-        fac.set_notes(model, NOTES.format(text))
+        fac.set_notes(model, NOTES.format(text), format=NotesFormat.HTML)
 
         old_line = None
         for line in lines:
@@ -360,7 +362,7 @@ def xpp2sbml(
                     continue
 
                 # necessary to find the additional arguments from the ast_node
-                ast = libsbml.parseL3Formula(formula)
+                ast: libsbml.ASTNode = libsbml.parseL3Formula(formula)
                 names = set(xpp_helpers.find_names_in_ast(ast))
                 old_args = [t.strip() for t in args.split(",")]
                 new_args = [a for a in names if a not in old_args]
@@ -558,7 +560,7 @@ def xpp2sbml(
                             trigger = g[1] + ">= 0"
 
                         assignment_parts = [t.strip() for t in g[2].split(";")]
-                        assignments = {}
+                        assignments: Dict[str, str] = {}
                         for p in assignment_parts:
                             key, value = p.split("=")
                             assignments[key] = value
@@ -567,7 +569,7 @@ def xpp2sbml(
                             fac.Event(
                                 sid="e{}".format(len(events)),
                                 trigger=trigger,
-                                assignments=assignments,
+                                assignments=assignments,  # type: ignore
                             )
                         )
 
@@ -614,15 +616,17 @@ def xpp2sbml(
     )
 
     # create SBML objects
-    objects = (
-        parameters
-        + initial_assignments
-        + functions
-        + rate_rules
-        + assignment_rules
-        + events
+    objects: List[Any] = list(
+        itertools.chain(
+            parameters,
+            initial_assignments,
+            functions,
+            rate_rules,
+            assignment_rules,
+            events,
+        )
     )
-    fac.create_objects(model, obj_iter=objects, debug=False)
+    fac.create_objects(model, obj_iter=objects)
 
     """
     Parameter values are optional; if not they are set to zero in xpp.
@@ -636,7 +640,6 @@ def xpp2sbml(
         doc,
         sbml_file,
         validate=validate,
-        program_name="sbmlutils",
-        program_version=__version__,
         units_consistency=False,
     )
+    return doc
