@@ -1,11 +1,12 @@
 """Helper functions for working with FBC and cobrapy models."""
-import logging
 import warnings
 
 import libsbml
 
+from sbmlutils import log
 
-logger = logging.getLogger(__name__)
+
+logger = log.get_logger(__name__)
 
 
 def set_flux_bounds(reaction: libsbml.Reaction, lb: float, ub: float) -> None:
@@ -17,7 +18,7 @@ def set_flux_bounds(reaction: libsbml.Reaction, lb: float, ub: float) -> None:
 
 def add_default_flux_bounds(
     doc: libsbml.SBMLDocument, lower: float = -100.0, upper: float = 100.0
-):
+) -> None:
     """Add default flux bounds to SBMLDocument.
 
     :param doc: SBMLDocument
@@ -54,7 +55,7 @@ def add_default_flux_bounds(
             rfbc.setUpperFluxBound("upper")
 
 
-def no_boundary_conditions(doc: libsbml.SBMLDocument):
+def no_boundary_conditions(doc: libsbml.SBMLDocument) -> None:
     """Set all boundaryCondition to False in the model.
 
     :param doc: libsbml.SBMLDocument
