@@ -10,29 +10,25 @@ model components.
 A standard workflow is looking up the components for instance in things like OLS
 ontology lookup service.
 """
-import logging
 import os
 import re
-from collections import OrderedDict
 from pathlib import Path
-from typing import Dict, Iterable, List, Union
+from typing import Dict, Iterable, List, Optional, Tuple, Union
 
 import libsbml
 import pandas as pd
+from pymetadata.core.annotation import RDFAnnotation as Annotation
+from pymetadata.identifiers.miriam import BQB, BQM
 
 from sbmlutils import utils
+from sbmlutils.console import console
 from sbmlutils.io.sbml import read_sbml, write_sbml
+from sbmlutils.log import get_logger
 
-from .miriam import (
-    BQB,
-    BQM,
-    IDENTIFIERS_ORG_PATTERN,
-    IDENTIFIERS_ORG_PREFIX,
-    MIRIAM_COLLECTION,
-)
+from ..validation import check
 
 
-LOGGER = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 def annotate_sbml(
@@ -46,7 +42,7 @@ def annotate_sbml(
     :param filepath: annotated SBML file
     :return: annotated SBMLDocument
     """
-    doc = read_sbml(source=source)
+    doc: libsbml.SBMLDocument = read_sbml(source=source)
 
     # annotate
     if not os.path.exists(str(annotations_path)):
@@ -54,10 +50,12 @@ def annotate_sbml(
     external_annotations = ModelAnnotator.read_annotations(
         annotations_path, file_format="*"
     )
-    doc = annotate_sbml_doc(doc, external_annotations)  # type: libsbml.SBMLDocument
+    doc = annotate_sbml_doc(doc, external_annotations)  # type: ignore
 
     # write annotated sbml
     write_sbml(doc, filepath=filepath)
+
+    console.print(f"Model annotated: file://{filepath}", style="success")
     return doc
 
 
@@ -75,151 +73,6 @@ def annotate_sbml_doc(
     return doc
 
 
-class Annotation:
-    """Annotation class.
-
-    Basic storage of annotation information. This consists of the relation
-    and the the resource.
-    The annotations can be attached to other objects thereby forming
-    triples which can be converted to RDF.
-
-    Resource can be either:
-        - `http(s)://identifiers.org/collection/term`, i.e., a identifiers.org URI
-        - `collection/term`, i.e., the combination of collection and term
-        - `http(s)://arbitrary.url`, an arbitrary URL
-
-    TODO: load the xrefs, synonyms and definitions from OLS
-    """
-
-    def __init__(self, qualifier, resource):
-        """Initialize annotation.
-
-        :param qualifier: BQM or BQB term
-        :param resource:
-        """
-        if not qualifier:
-            raise ValueError(
-                "MIRIAM qualifiers are required for annotation, but no "
-                "qualifier for resource '{resource}' was provided."
-            )
-        if not resource:
-            raise ValueError(
-                f"resource is required for annotation, but resource is emtpy "
-                f"'{qualifier} {resource}'."
-            )
-        if not isinstance(resource, str):
-            raise ValueError(
-                f"resource must be string, but found '{resource} {type(resource)}'."
-            )
-
-        self.qualifier = qualifier
-        self.collection = None
-        self.term = None
-
-        # handle urls
-        if resource.startswith("http"):
-            match = IDENTIFIERS_ORG_PATTERN.match(resource)
-            if match:
-                # handle identifiers.org pattern
-                self.collection, self.term = match.group(1), match.group(2)
-            else:
-                # other urls are directly stored as resources without collection
-                self.collection = None
-                self.term = resource
-                LOGGER.warning(
-                    "%s does not conform to " "http(s)://identifiers.org/collection/id",
-                    resource,
-                )
-        else:
-            # get term and collection
-            tokens = resource.split("/")
-            if len(tokens) < 2:
-                LOGGER.error(
-                    "Resource `{}` could not be split in collection and term. "
-                    "A given resource must be of the form "
-                    "`collection/term` or an url starting with "
-                    "`http(s)://`)".format(resource)
-                )
-                self.collection = None
-                self.term = resource
-            else:
-                self.collection = tokens[0]
-                self.term = "/".join(tokens[1:])
-
-        self.validate()
-
-    @staticmethod
-    def from_tuple(t):
-        """Construct from tuple."""
-        qualifier, resource = t[0], t[1]
-        return Annotation(qualifier=qualifier, resource=resource)
-
-    @property
-    def resource(self):
-        """Resource for annotations."""
-        if self.collection:
-            return "{}/{}/{}".format(IDENTIFIERS_ORG_PREFIX, self.collection, self.term)
-        else:
-            return self.term
-
-    def to_dict(self) -> Dict:
-        """Convert to dictionary."""
-        return {
-            "qualifier": self.qualifier.value,
-            "collection": self.collection,
-            "term": self.term,
-            "resource": self.resource,
-        }
-
-    @staticmethod
-    def check_term(collection, term):
-        """Check that a given term follows id pattern for existing collection.
-
-        :param collection:
-        :param term:
-        :return:
-        """
-        entry = MIRIAM_COLLECTION.get(collection, None)
-        if not entry:
-            logging.error(
-                "The given MIRIAM collection `{}` in annotation"
-                "`{}/{}` does not exist.".format(collection, collection, term)
-            )
-            return False
-
-        p = re.compile(entry["pattern"])
-        m = p.match(term)
-        if not m:
-            logging.error(
-                "Term `{}` did not match pattern "
-                "`{}` for collection `{}`.".format(term, entry["pattern"], collection)
-            )
-            return False
-
-        return True
-
-    @staticmethod
-    def check_qualifier(qualifier):
-        """Check that the qualifier is an allowed qualifier.
-
-        :param qualifier:
-        :return:
-        """
-        if not isinstance(qualifier, (BQB, BQM)):
-            supported_qualifiers = [e.value for e in BQB] + [e.value for e in BQM]
-
-            raise ValueError(
-                f"qualifier '{qualifier}' is not in supported qualifiers: "
-                f"{supported_qualifiers}"
-            )
-
-    def validate(self):
-        """Validate annotation."""
-        self.check_qualifier(self.qualifier)
-        if self.collection:
-            self.check_term(collection=self.collection, term=self.term)
-
-
 class ExternalAnnotation:
     """Class for handling SBML annotations defined in external source.
 
@@ -255,9 +108,16 @@ class ExternalAnnotation:
     )
     _annotation_types = frozenset(["rdf", "formula", "charge"])
 
-    def __init__(self, d):
+    def __init__(self, d: Dict):
         """Initialize ExternalAnnotation."""
         self.d = d
+        self.pattern: Optional[str] = None
+        self.sbml_type: Optional[str] = None
+        self.annotation_type: Optional[str] = None
+        self.qualifier: Optional[Union[BQB, BQM]] = None
+        self.resource: Optional[str] = None
+        self.name: Optional[str] = None
+
         for key in self._keys:
             # optional fields
             if key in ["qualifier", "name"]:
@@ -273,29 +133,33 @@ class ExternalAnnotation:
             setattr(self, key, value)
 
         if self.annotation_type == "rdf":
-            self.qualifier = ExternalAnnotation._parse_qualifier(self.qualifier)
+            self.qualifier = ExternalAnnotation._parse_qualifier_str(self.qualifier)  # type: ignore
         else:
             self.qualifier = None
 
         self.check()
 
     @staticmethod
-    def _parse_qualifier(qualifier):
+    def _parse_qualifier_str(qualifier: Optional[str]) -> Union[BQB, BQM]:
+        if qualifier is None:
+            raise ValueError("Qualifier must be provided.")
+
         if not qualifier.startswith("BQ"):
-            raise ValueError(
-                "Qualifier must start with BQM_ or BQB_: `{}`".format(qualifier)
-            )
-        bq = None
+            raise ValueError(f"Qualifier must start with BQM_ or BQB_: '{qualifier}'")
+        bq: Union[BQB, BQM]
         if qualifier.startswith("BQM_"):
             bq = BQM[qualifier[4:]]
         elif qualifier.startswith("BQB_"):
             bq = BQB[qualifier[4:]]
         if bq is None:
-            raise ValueError("Qualifier could not be parsed: `{}`".format(qualifier))
+            raise ValueError(f"Qualifier could not be parsed: '{qualifier}'")
         return bq
 
-    def check(self):
-        """Check for valid choices."""
+    def check(self) -> None:
+        """Check for valid choices.
+
+        :raise: ValueError
+        """
         if self.sbml_type not in self._sbml_types:
             raise ValueError(
                 f"Invalid sbml_type '{self.sbml_type}'. "
@@ -332,7 +196,7 @@ class ModelAnnotator:
         # prepare dictionary for lookup of ids
         self.id_dict = self._get_ids_from_model()
 
-    def annotate_model(self):
+    def annotate_model(self) -> None:
         """Annotate the model with the given annotations."""
         # writes all annotations
         for a in self.annotations:
@@ -341,18 +205,23 @@ class ModelAnnotator:
                 elements = [self.doc]
             else:
                 # lookup of allowed ids for given sbmlutils type
-                ids = self.id_dict.get(a.sbml_type, None)
+                ids = self.id_dict.get(a.sbml_type, None)  # type: ignore
                 elements = []
                 if ids:
                     # find the subset of ids matching the pattern
-                    pattern_ids = ModelAnnotator._get_matching_ids(ids, pattern)
+                    pattern_ids = ModelAnnotator._get_matching_ids(ids, pattern)  # type: ignore
+                    if not pattern_ids:
+                        logger.warning(
+                            f"No SBML objects found matching SId annotation "
+                            f"pattern: '{pattern}'"
+                        )
                     elements = ModelAnnotator._elements_from_ids(
                         self.model, pattern_ids, sbml_type=a.sbml_type
                     )
 
             self._annotate_elements(elements, a)
 
-    def _get_ids_from_model(self):
+    def _get_ids_from_model(self) -> Dict[str, List[str]]:
         """Create dictionary of ids for given model for lookup.
 
         :return:
@@ -426,12 +295,14 @@ class ModelAnnotator:
                 if sid == model.getId():
                     e = model
                 else:
-                    logging.warning(f"Element not found for sid: '{sid}'.")
+                    logger.warning(f"Element not found for sid: '{sid}'.")
                     continue
             elements.append(e)
         return elements
 
-    def _annotate_elements(self, elements, ex_a: ExternalAnnotation):
+    def _annotate_elements(
+        self, elements: Iterable[libsbml.SBase], ex_a: ExternalAnnotation
+    ) -> None:
         """Annotate given elements with annotation.
 
         :param elements: SBase elements to annotate
@@ -441,7 +312,7 @@ class ModelAnnotator:
         for e in elements:
             if ex_a.annotation_type == "rdf":
                 annotation = Annotation(
-                    qualifier=ex_a.qualifier, resource=ex_a.resource
+                    qualifier=ex_a.qualifier, resource=ex_a.resource  # type: ignore
                 )
                 ModelAnnotator.annotate_sbase(e, annotation)
 
@@ -452,75 +323,116 @@ class ModelAnnotator:
             elif ex_a.annotation_type in ["formula", "charge"]:
                 # via fbc species plugin, so check that species first
                 if ex_a.sbml_type != "species":
-                    LOGGER.error(
+                    logger.error(
                         "Chemical formula or Charge can only be " "set on species."
                     )
                 else:
                     s = self.model.getSpecies(e.getId())
                     splugin = s.getPlugin("fbc")
                     if splugin is None:
-                        LOGGER.error(
-                            "FBC SPlugin not found for species, " "no fbc: {}".format(s)
+                        logger.error(
+                            "FbcSpeciesPlugin does not exist, add `packages = ['fbc']`"
+                            " to model definition."
                         )
                     else:
                         if ex_a.annotation_type == "formula":
                             splugin.setChemicalFormula(ex_a.resource)
                         elif ex_a.annotation_type == "charge":
-                            splugin.setCharge(int(ex_a.resource))
+                            splugin.setCharge(int(ex_a.resource))  # type: ignore
             else:
                 raise ValueError(
                     "Annotation type not supported: " "{}".format(ex_a.annotation_type)
                 )
 
     @staticmethod
-    def get_SBMLQualifier(qualifier_str):
-        """Lookup of SBMLQualifier for given qualifier string."""
+    def get_SBMLQualifier(qualifier_str: str, qualifier_type: str) -> str:
+        """Lookup of SBMLQualifier for given qualifier string.
 
-        # FIXME: better lookup with MIRIAM
+        :param qualifier_type: BQB or BQM
+        :return: SBML qualifier string
+        """
         if qualifier_str not in libsbml.__dict__:
-            raise ValueError("Qualifier not supported: {}".format(qualifier_str))
-        return libsbml.__dict__.get(qualifier_str)
+            raise ValueError(f"Qualifier not supported: {qualifier_str}")
+
+        qtype = libsbml.__dict__.get(qualifier_str)
+
+        qualifier: str
+        if qualifier_type == "BQB":
+            qualifier = str(libsbml.BiolQualifierType_toString(qtype))
+        elif qualifier_type == "BQM":
+            qualifier = str(libsbml.ModelQualifierType_toString(qtype))
+        return qualifier
 
     @staticmethod
-    def annotate_sbase(sbase: libsbml.SBase, annotation: Annotation):
+    def annotate_sbase(sbase: libsbml.SBase, annotation: Annotation) -> None:
         """Annotate SBase based on given annotation data.
 
         :param sbase: libsbml.SBase
         :param annotation: Annotation
         :return:
         """
-        qualifier, resource = annotation.qualifier.value, annotation.resource
-        cv = libsbml.CVTerm()  # type: libsbml.CVTerm
+        qualifier, resource = annotation.qualifier.value, annotation.resource_normalized
+        cv: libsbml.CVTerm = libsbml.CVTerm()
 
         # set correct type of qualifier
-        if qualifier.startswith("BQB"):
-            cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
-            sbml_qualifier = ModelAnnotator.get_SBMLQualifier(qualifier)
-            cv.setBiologicalQualifierType(sbml_qualifier)
-        elif qualifier.startswith("BQM"):
-            cv.setQualifierType(libsbml.MODEL_QUALIFIER)
-            sbml_qualifier = ModelAnnotator.get_SBMLQualifier(qualifier)
-            cv.setModelQualifierType(sbml_qualifier)
-        else:
-            LOGGER.error("Unsupported qualifier: {}".format(qualifier))
+        if isinstance(qualifier, str):
+            if qualifier.startswith("BQB"):
+                cv.setQualifierType(libsbml.BIOLOGICAL_QUALIFIER)
 
-        cv.addResource(resource)
+                sbml_qualifier = ModelAnnotator.get_SBMLQualifier(qualifier, "BQB")
+                success = check(
+                    cv.setBiologicalQualifierType(sbml_qualifier),
+                    f"Set biological qualifier: '{sbml_qualifier}'",
+                )
+                if not success:
+                    logger.error(
+                        f"Could not set biological qualifier '{qualifier}' "
+                        f"for '{sbase}'."
+                    )
+            elif qualifier.startswith("BQM"):
+                cv.setQualifierType(libsbml.MODEL_QUALIFIER)
+                sbml_qualifier = ModelAnnotator.get_SBMLQualifier(qualifier, "BQM")
+                success = check(
+                    cv.setModelQualifierType(sbml_qualifier),
+                    f"Set model qualifier: '{sbml_qualifier}'",
+                )
+                if not success:
+                    logger.error(
+                        f"Could not set model qualifier '{qualifier}' "
+                        f"for '{sbase}'."
+                    )
+            else:
+                logger.error(f"Unsupported qualifier: '{qualifier}' for '{sbase}'.")
+        else:
+            msg = (
+                f"qualifier is not a string, but: '{qualifier}' of type "
+                f"'{type(qualifier)}' for '{sbase}'."
+            )
+            logger.error(msg)
+            raise ValueError(msg)
+
+        success = check(cv.addResource(resource), f"Add resource: '{resource}'.")
+        if not success:
+            logger.error(f"Could not add resource: {resource} for '{sbase}'.")
 
         # meta id has to be set
         if not sbase.isSetMetaId():
             sbase.setMetaId(utils.create_metaid(sbase))
 
-        success = sbase.addCVTerm(cv)
+        success = check(sbase.addCVTerm(cv), f"Add cvterm: '{cv}'.")
 
-        if success != 0:
-            LOGGER.error("RDF not written: ", success)
-            LOGGER.error(libsbml.OperationReturnValue_toString(success))
-            LOGGER.error("{}, {}, {}".format(object, qualifier, resource))
+        if not success:
+            logger.error(
+                f"Annotation RDF for CVTerm '{cv}' could not be written "
+                f"for '{sbase}'."
+            )
+            logger.error(libsbml.OperationReturnValue_toString(success))
+            logger.error(f"{sbase}, {qualifier}, {resource}")
 
     # --- File IO ---
 
     @staticmethod
-    def read_annotations_df(file_path: Path, file_format: str = "*"):
+    def read_annotations_df(file_path: Path, file_format: str = "*") -> pd.DataFrame:
         """Read annotations from given file into DataFrame.
 
         Supports "xlsx", "tsv", "csv", "json", "*"
@@ -542,7 +454,7 @@ class ModelAnnotator:
             )
 
         if file_extension != ("." + file_format):
-            logging.warning(
+            logger.warning(
                 "format '{}' not matching file extension '{}' for file_path '{}'".format(
                     file_format, file_extension, file_path
                 )
@@ -562,7 +474,7 @@ class ModelAnnotator:
 
     @staticmethod
     def read_annotations(
-        file_path: [Path, Dict], file_format: str = "*"
+        file_path: Path, file_format: str = "*"
     ) -> List[ExternalAnnotation]:
         """Read annotations from given file into DataFrame.
 
