@@ -1,3 +1,5 @@
+from pathlib import Path
+
 from sbmlutils.io.sbml import validate_sbml
 from sbmlutils.test import BASIC_SBML, DEMO_SBML, GALACTOSE_SINGLECELL_SBML, VDP_SBML
 
@@ -10,7 +12,9 @@ SBML_FILES = [
 ]
 
 
-def _validate_file(sbmlpath, units_consistency=True, Nall=0):
+def _validate_file(
+    sbmlpath: Path, units_consistency: bool = True, Nall: int = 0
+) -> None:
     """Validate given SBML file.
 
     Helper function called by the other tests.
@@ -24,6 +28,6 @@ def _validate_file(sbmlpath, units_consistency=True, Nall=0):
     assert Nall == v_results.all_count
 
 
-def test_files():
+def test_files() -> None:
     for d in SBML_FILES:
-        _validate_file(sbmlpath=d["path"], units_consistency=d["ucheck"], Nall=d["N"])
+        _validate_file(sbmlpath=d["path"], units_consistency=d["ucheck"], Nall=d["N"])  # type: ignore
